@@ -501,6 +501,7 @@ vAPI.DOMFilterer = class {
         this.stylesheets = [];
         this.exceptedCSSRules = [];
         this.exceptions = [];
+        this.convertedProceduralFilters = [];
         this.proceduralFilterer = null;
         // https://github.com/uBlockOrigin/uBlock-issues/issues/167
         //   By the time the DOMContentLoaded is fired, the content script might
@@ -518,11 +519,11 @@ vAPI.DOMFilterer = class {
 
     explodeCSS(css) {
         const out = [];
-        const reBlock = /^\{(.*)\}$/m;
+        const cssHide = `{${vAPI.hideStyle}}`;
         const blocks = css.trim().split(/\n\n+/);
         for ( const block of blocks ) {
-            const match = reBlock.exec(block);
-            out.push([ block.slice(0, match.index).trim(), match[1] ]);
+            if ( block.endsWith(cssHide) === false ) { continue; }
+            out.push(block.slice(0, -cssHide.length).trim());
         }
         return out;
     }
@@ -621,9 +622,6 @@ vAPI.DOMFilterer = class {
     }
 
     addProceduralSelectors(selectors) {
-        if ( Array.isArray(selectors) === false || selectors.length === 0 ) {
-            return;
-        }
         const procedurals = [];
         for ( const raw of selectors ) {
             procedurals.push(JSON.parse(raw));
@@ -652,23 +650,30 @@ vAPI.DOMFilterer = class {
             ? `[${this.proceduralFilterer.masterToken}]`
             : undefined;
         for ( const css of this.stylesheets ) {
-            const blocks = this.explodeCSS(css);
-            for ( const block of blocks ) {
+            for ( const block of this.explodeCSS(css) ) {
                 if (
                     includePrivateSelectors === false &&
                     masterToken !== undefined &&
-                    block[0].startsWith(masterToken)
+                    block.startsWith(masterToken)
                 ) {
                     continue;
                 }
-                out.declarative.push([ block[0], block[1] ]);
+                out.declarative.push(block);
             }
         }
         const excludeProcedurals = (bits & 0b10) !== 0;
-        if ( excludeProcedurals !== true ) {
-            out.procedural = hasProcedural
-                ? Array.from(this.proceduralFilterer.selectors.values())
-                : [];
+        if ( excludeProcedurals === false ) {
+            out.procedural = [];
+            if ( hasProcedural ) {
+                out.procedural.push(
+                    ...this.proceduralFilterer.selectors.values()
+                );
+            }
+            for ( const json of this.convertedProceduralFilters ) {
+                out.procedural.push(
+                    this.proceduralFiltererInstance().createProceduralFilter(json)
+                );
+            }
         }
         return out;
     }
@@ -1024,7 +1029,6 @@ vAPI.DOMFilterer = class {
     // Extract all classes/ids: these will be passed to the cosmetic
     // filtering engine, and in return we will obtain only the relevant
     // CSS selectors.
-    const reWhitespace = /\s/;
 
     // https://github.com/gorhill/uBlock/issues/672
     // http://www.w3.org/TR/2014/REC-html5-20141028/infrastructure.html#space-separated-tokens
@@ -1039,19 +1043,21 @@ vAPI.DOMFilterer = class {
         queriedIds.add(s);
     };
 
+    // https://github.com/uBlockOrigin/uBlock-issues/discussions/2076
+    //   Performance: avoid using Element.classList
     const classesFromNode = (node, out) => {
-        const s = node.className;
-        if ( typeof s !== 'string' || s.length === 0 ) { return; }
-        if ( reWhitespace.test(s) === false ) {
-            if ( queriedClasses.has(s) ) { return; }
-            out.push(s);
-            queriedClasses.add(s);
-            return;
-        }
-        for ( const s of node.classList.values() ) {
-            if ( queriedClasses.has(s) ) { continue; }
-            out.push(s);
-            queriedClasses.add(s);
+        const s = node.getAttribute('class');
+        if ( typeof s !== 'string' ) { return; }
+        const len = s.length;
+        for ( let beg = 0, end = 0, token = ''; beg < len; beg += 1 ) {
+            end = s.indexOf(' ', beg);
+            if ( end === beg ) { continue; }
+            if ( end === -1 ) { end = len; }
+            token = s.slice(beg, end);
+            beg = end;
+            if ( queriedClasses.has(token) ) { continue; }
+            out.push(token);
+            queriedClasses.add(token);
         }
     };
 
@@ -1318,6 +1324,7 @@ vAPI.DOMFilterer = class {
             domFilterer.addCSS(cfeDetails.injectedCSS);
             domFilterer.addProceduralSelectors(cfeDetails.proceduralFilters);
             domFilterer.exceptCSSRules(cfeDetails.exceptedFilters);
+            domFilterer.convertedProceduralFilters = cfeDetails.convertedProceduralFilters;
         }
 
         vAPI.userStylesheet.apply();
