@@ -1263,6 +1263,7 @@ const pickerCSSStyle = [
     'border-radius: 0',
     'box-shadow: none',
     'display: block',
+    'filter: none',
     'height: 100vh',
     'left: 0',
     'margin: 0',
@@ -1276,22 +1277,15 @@ const pickerCSSStyle = [
     'pointer-events: auto',
     'position: fixed',
     'top: 0',
+    'transform: none',
     'visibility: visible',
     'width: 100%',
     'z-index: 2147483647',
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/1408
+    'color-scheme: light',
     ''
 ];
 
-// https://github.com/uBlockOrigin/uBlock-issues/issues/1408
-//   We need to reset Chromium-specific `color-scheme` property
-//   for our iframe widget.
-if (
-    CSS.supports instanceof Function &&
-    CSS.supports('color-scheme', 'initial')
-) {
-    pickerCSSStyle.push('color-scheme: initial');
-}
-
 const pickerCSS = `
 :root > [${vAPI.sessionId}] {
     ${pickerCSSStyle.join(' !important;')}
